@@ -25,17 +25,17 @@ use warnings;
 
 use Carp qw(croak);
 use Cwd();
-use Data::Dumper;
+use File::Temp qw(tempdir);
 use Time::HiRes qw(gettimeofday tv_interval);
+use Path::Tiny;
 use POSIX qw(:sys_wait_h);
 
 use Lintian::DepMap;
 use Lintian::DepMap::Properties;
-use Lintian::Output qw(:messages);
 use Lintian::Processable::Group;
-use Lintian::Unpacker;
 use Lintian::Util;
 
+use constant EMPTY => q{};
 use constant SPACE => q{ };
 
 use Moo;
@@ -68,19 +68,35 @@ Lintian::Processable::Pool -- Pool of processables
 Returns a hash reference to the list of processable groups that are currently
 in the pool. The key is a unique identifier based on name and version.
 
-=item $pool->lab
-
-Returns or sets the lab used by this pool.
-
-=item $pool->unpacker
-
-Returns or sets the Unpacker object used by this pool.
-
 =cut
 
 has groups => (is => 'rwp', default => sub{ {} });
-has lab => (is => 'rw', default => sub { Lintian::Lab->new });
-has unpacker => (is => 'rwp');
+
+# must be absolute; frontend/lintian depends on it
+has basedir => (
+    is => 'rwp',
+    default => sub {
+
+        my $relative = tempdir('temp-lintian-lab-XXXXXXXXXX', 'TMPDIR' => 1);
+
+        my $absolute = Cwd::abs_path($relative);
+        croak "Could not resolve $relative: $!"
+          unless $absolute;
+
+        path("$absolute/pool")->mkpath({mode => 0777});
+
+        return $absolute;
+    });
+has keep => (is => 'rw', default => 0);
+
+=item $pool->basedir
+
+Returns the base directory for the pool. Most likely it's a temporary directory.
+
+=item $pool->keep
+
+Returns or accepts a boolean value that indicates whether the lab should be
+removed when Lintian finishes. Used for debugging.
 
 =item $pool->add_group($group)
 
@@ -132,8 +148,8 @@ Process the pool.
 sub process{
     my (
         $self, $action,$PROFILE,$TAGS,
-        $exit_code_ref, $overrides,$opt,$memory_usage,
-        $STATUS_FD, $unpack_info_ref
+        $exit_code_ref, $opt,$memory_usage,$STATUS_FD,
+        $unpack_info_ref, $OUTPUT
     ) = @_;
 
     # $map is just here to check that all the needed collections are present.
@@ -152,7 +168,8 @@ sub process{
 
         my $cs = Lintian::CollScript->new("$dirname/$file");
 
-        debug_msg(2, 'Read collector description for ' . $cs->name . '...');
+        $OUTPUT->debug_msg(2,
+            'Read collector description for ' . $cs->name . '...');
         $collmap->add($cs->name, $cs->needs_info, $cs);
         $map->addp('coll-' . $cs->name, 'coll-', $cs->needs_info);
     }
@@ -161,7 +178,7 @@ sub process{
       or warn 'Close failed';
 
     my @scripts = sort $PROFILE->scripts;
-    debug_msg(
+    $OUTPUT->debug_msg(
         1,
         "Selected action: $action",
         sprintf('Selected checks: %s', join(',', @scripts)),
@@ -199,15 +216,11 @@ sub process{
               . join(', ', $map->missing));
     }
 
-    my $unpacker = Lintian::Unpacker->new;
-
-    # for checking, pass profile to limit what it unpacks
+    my @requested;
     if ($action eq 'check') {
 
-        $unpacker->profile($PROFILE);
-
         # add collections requested by user (--unpack-info)
-        my @requested
+        @requested
           = map { split(/,/) } (@{$unpack_info_ref // []});
 
         my @unknown = grep { !collmap->getp($_) } @requested;
@@ -217,42 +230,43 @@ sub process{
         # need 'override-file' for overrides
         push(@requested, 'override-file')
           unless $opt->{'no-override'};
-
-        $unpacker->extra_coll(\@requested);
     }
 
     # With --unpack we want all of them.  That's the default so,
     # "done!"
 
-    $unpacker->jobs($opt->{'jobs'});
-    $unpacker->init($collmap);
-    $self->_set_unpacker($unpacker);
+    my %override_count;
 
     my @sorted = sort { $a->name cmp $b->name } values %{$self->groups};
     foreach my $group (@sorted) {
         my $success = 1;
 
-        v_msg('Starting on group ' . $group->name);
+        $OUTPUT->v_msg('Starting on group ' . $group->name);
 
         my $total_start = [gettimeofday];
         my $group_start = [gettimeofday];
 
-        if (!$group->unpack($self->unpacker, $action,$exit_code_ref)) {
+        # for checking, pass profile to limit what it unpacks
+        $group->profile($PROFILE);
+
+        $group->extra_coll(\@requested);
+        $group->jobs($opt->{'jobs'});
+
+        if (!$group->unpack($collmap, $action,$exit_code_ref, $OUTPUT)) {
             $success = 0;
         }
 
         my $raw_res = tv_interval($group_start);
         my $tres = sprintf('%.3fs', $raw_res);
 
-        debug_msg(1, 'Unpack of ' . $group->name . " done ($tres)");
-        perf_log($group->name . ",total-group-unpack,${raw_res}");
+        $OUTPUT->debug_msg(1, 'Unpack of ' . $group->name . " done ($tres)");
+        $OUTPUT->perf_log($group->name . ",total-group-unpack,${raw_res}");
 
         if ($action eq 'check') {
             if (
                 !$group->process(
-                    $PROFILE,$TAGS, $collmap,
-                    $exit_code_ref, $overrides,$opt,
-                    $memory_usage
+                    $TAGS,$exit_code_ref, \%override_count,
+                    $opt,$memory_usage, $OUTPUT
                 )
             ) {
                 $success = 0;
@@ -284,8 +298,15 @@ sub process{
         }
 
         # remove group files unless we are keeping the lab
-        $group->clean_lab
-          unless ($self->lab->keep);
+        $group->clean_lab($OUTPUT)
+          unless ($self->keep);
+
+       # Wait for any remaining jobs - There will usually not be any
+       # unless we had an issue examining the last package.  We patiently wait
+       # for them here; if the user cannot be bothered to wait, he/she can send
+       # us a signal and the END handler will kill any remaining jobs.
+
+        $group->wait_for_jobs;
 
         my $total_raw_res = tv_interval($total_start);
         my $total_tres = sprintf('%.3fs', $total_raw_res);
@@ -295,34 +316,72 @@ sub process{
         } else {
             print {$STATUS_FD} 'error ' . $group->name . " ($total_tres)\n";
         }
-        v_msg('Finished processing group ' . $group->name);
+        $OUTPUT->v_msg('Finished processing group ' . $group->name);
     }
 
-    # Wait for any remaining jobs - There will usually not be any
-    # unless we had an issue examining the last package.  We patiently wait
-    # for them here; if the user cannot be bothered to wait, he/she can send
-    # us a signal and the END handler will kill any remaining jobs.
-
-    $self->unpacker->wait_for_jobs;
-
     # do not remove lab if so selected
-    $self->lab->keep($opt->{'keep-lab'});
+    $self->keep($opt->{'keep-lab'});
 
-    return;
-}
+    if (    $action eq 'check'
+        and not $opt->{'no-override'}
+        and not $opt->{'show-overrides'}) {
 
-=item DEMOLISH
+        my $errors = $override_count{errors} || 0;
+        my $warnings = $override_count{warnings} || 0;
+        my $info = $override_count{info} || 0;
+        my $total = $errors + $warnings + $info;
 
-Moo destructor.
+        my $unused = $override_count{unused} || 0;
 
-=cut
+        if ($total > 0 or $unused > 0) {
+            my $text
+              = ($total == 1)
+              ? "$total tag overridden"
+              : "$total tags overridden";
+            my @output;
+            if ($errors) {
+                push(@output,
+                    ($errors == 1) ? "$errors error" : "$errors errors");
+            }
+            if ($warnings) {
+                push(@output,
+                    ($warnings == 1)
+                    ? "$warnings warning"
+                    : "$warnings warnings");
+            }
+            if ($info) {
+                push(@output, "$info info");
+            }
+            if (@output) {
+                $text .= ' (' . join(', ', @output). ')';
+            }
+            if ($unused == 1) {
+                $text .= "; $unused unused override";
+            } elsif ($unused > 1) {
+                $text .= "; $unused unused overrides";
+            }
+            $OUTPUT->msg($text);
+        }
+    }
 
-sub DEMOLISH {
-    my ($self, $in_global_destruction) = @_;
-
-    # kill any remaining jobs.
-    $self->unpacker->kill_jobs
-      if $self->unpacker;
+    my $ign_over = $override_count{ignored};
+    if (keys %$ign_over) {
+        $OUTPUT->msg(
+            join(q{ },
+                'Some overrides were ignored,',
+                'since the tags were marked "non-overridable".'));
+        if ($opt->{'verbose'}) {
+            $OUTPUT->v_msg(
+                join(q{ },
+                    'The following tags were "non-overridable"',
+                    'and had at least one override'));
+            foreach my $tag (sort keys %$ign_over) {
+                $OUTPUT->v_msg("  - $tag");
+            }
+        } else {
+            $OUTPUT->msg('Use --verbose for more information.');
+        }
+    }
 
     return;
 }
@@ -360,7 +419,23 @@ Returns true if the pool is empty.
 
 sub empty{
     my ($self) = @_;
-    return scalar %{$self->groups} == 0;
+    return scalar keys %{$self->groups} == 0;
+}
+
+=item DEMOLISH
+
+Removes the lab and everything in it.  Any reference to an entry
+returned from this lab will immediately become invalid.
+
+=cut
+
+sub DEMOLISH {
+    my ($self, $in_global_destruction) = @_;
+
+    path($self->basedir)->remove_tree
+      if length $self->basedir && -d $self->basedir && !$self->keep;
+
+    return;
 }
 
 =back
